import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode
from pathlib import Path
from tgram.utils import get_file_path, get_parse_mode, convert_to_inline_keyboard_markup


class SendAnimation:
    async def send_animation(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        animation: Union[Path, bytes, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        duration: int = None,
        width: int = None,
        height: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send animation files (GIF or H.264/MPEG-4 AVC video without sound).
        On success, the sent Message is returned. Bots can currently send animation files of up to 50 MB in size, this limit may be changed in the future.

        Telegram documentation: https://core.telegram.org/bots/api#sendanimation

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param animation: Animation to send. Pass a file_id as String to send an animation that exists on the Telegram servers (recommended),
            pass an HTTP URL as a String for Telegram to get an animation from the Internet, or upload a new animation using multipart/form-data.
        :type animation: :obj:`str` or :class:`tgram.types.InputFile`

        :param duration: Duration of sent animation in seconds
        :type duration: :obj:`int`

        :param width: Animation width
        :type width: :obj:`int`

        :param height: Animation height
        :type height: :obj:`int`

        :param thumbnail: Thumbnail of the file sent; can be ignored if thumbnail generation for the file is supported server-side.
            The thumbnail should be in JPEG format and less than 200 kB in size. A thumbnail's width and height should not exceed 320.
            Ignored if the file is not uploaded using multipart/form-data. Thumbnails can't be reused and can be only uploaded as a new file,
            so you can pass “attach://<file_attach_name>” if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
        :type thumbnail: :obj:`str` or :class:`tgram.types.InputFile`

        :param caption: Animation caption (may also be used when resending animation by file_id), 0-1024 characters after entities parsing
        :type caption: :obj:`str`

        :param parse_mode: Mode for parsing entities in the animation caption
        :type parse_mode: :obj:`str`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard
            or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param caption_entities: List of special entities that appear in the caption, which can be specified instead of parse_mode
        :type caption_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param message_thread_id: Identifier of a message thread, in which the video will be sent
        :type message_thread_id: :obj:`int`

        :param has_spoiler: Pass True, if the animation should be sent as a spoiler
        :type has_spoiler: :obj:`bool`

        :param thumb: Deprecated. Use thumbnail instead
        :type thumb: :obj:`str` or :class:`tgram.types.InputFile`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Identifier of a business connection, in which the message will be sent
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier of the message effect
        :type message_effect_id: :obj:`str`

        :param show_caption_above_media: Pass True, if the caption must be shown above the message media. Supported only for animation, photo and video messages.
        :type show_caption_above_media: :obj:`bool`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendAnimation",
            chat_id=chat_id,
            animation=get_file_path(animation),
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            duration=duration,
            width=width,
            height=height,
            thumbnail=get_file_path(thumbnail),
            caption=caption,
            parse_mode=get_parse_mode(self, parse_mode),
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            has_spoiler=has_spoiler,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_
from typing import List, Optional, Union


class OwnedGiftRegular(Type_):
    """
    Describes a regular gift owned by a user or a chat.

    :param type: Type of the gift, always "regular"
    :type type: :obj:`str`
    :param gift: Information about the regular gift
    :type gift: :class:`tgram.types.Gift`
    :param owned_gift_id: Optional. Unique identifier of the gift for the bot; for gifts received on behalf of business accounts only
    :type owned_gift_id: :obj:`str`
    :param sender_user: Optional. Sender of the gift if it is a known user
    :type sender_user: :class:`tgram.types.User`
    :param send_date: Date the gift was sent in Unix time
    :type send_date: :obj:`int`
    :param text: Optional. Text of the message that was added to the gift
    :type text: :obj:`str`
    :param entities: Optional. Special entities that appear in the text
    :type entities: List[:class:`tgram.types.MessageEntity`]
    :param is_private: Optional. True, if the sender and gift text are shown only to the gift receiver
    :type is_private: :obj:`bool`
    :param is_saved: Optional. True, if the gift is displayed on the account's profile page
    :type is_saved: :obj:`bool`
    :param can_be_upgraded: Optional. True, if the gift can be upgraded to a unique gift
    :type can_be_upgraded: :obj:`bool`
    :param was_refunded: Optional. True, if the gift was refunded and isn't available anymore
    :type was_refunded: :obj:`bool`
    :param convert_star_count: Optional. Number of Telegram Stars that can be claimed by the receiver instead of the gift
    :type convert_star_count: :obj:`int`
    :param prepaid_upgrade_star_count: Optional. Number of Telegram Stars that were paid by the sender for the ability to upgrade the gift
    :type prepaid_upgrade_star_count: :obj:`int`
    """

    def __init__(
        self,
        gift: "tgram.types.Gift" = None,
        owned_gift_id: str = None,
        sender_user: "tgram.types.User" = None,
        send_date: int = None,
        text: str = None,
        entities: List["tgram.types.MessageEntity"] = None,
        is_private: bool = None,
        is_saved: bool = None,
        can_be_upgraded: bool = None,
        was_refunded: bool = None,
        convert_star_count: int = None,
        prepaid_upgrade_star_count: int = None,
        me: "tgram.TgBot" = None,
        json: dict = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "regular"
        self.gift = gift
        self.owned_gift_id = owned_gift_id
        self.sender_user = sender_user
        self.send_date = send_date
        self.text = text
        self.entities = entities
        self.is_private = is_private
        self.is_saved = is_saved
        self.can_be_upgraded = can_be_upgraded
        self.was_refunded = was_refunded
        self.convert_star_count = convert_star_count
        self.prepaid_upgrade_star_count = prepaid_upgrade_star_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["OwnedGiftRegular"]:
        return (
            OwnedGiftRegular(
                gift=tgram.types.Gift._parse(me, d.get("gift")),
                owned_gift_id=d.get("owned_gift_id"),
                sender_user=tgram.types.User._parse(me, d.get("sender_user")),
                send_date=d.get("send_date"),
                text=d.get("text"),
                entities=[
                    tgram.types.MessageEntity._parse(me, ent)
                    for ent in d.get("entities")
                ]
                if d.get("entities")
                else None,
                is_private=d.get("is_private"),
                is_saved=d.get("is_saved"),
                can_be_upgraded=d.get("can_be_upgraded"),
                was_refunded=d.get("was_refunded"),
                convert_star_count=d.get("convert_star_count"),
                prepaid_upgrade_star_count=d.get("prepaid_upgrade_star_count"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class OwnedGiftUnique(Type_):
    """
    Describes a unique gift received and owned by a user or a chat.

    :param type: Type of the gift, always "unique"
    :type type: :obj:`str`
    :param gift: Information about the unique gift
    :type gift: :class:`tgram.types.UniqueGift`
    :param owned_gift_id: Optional. Unique identifier of the received gift for the bot; for gifts received on behalf of business accounts only
    :type owned_gift_id: :obj:`str`
    :param sender_user: Optional. Sender of the gift if it is a known user
    :type sender_user: :class:`tgram.types.User`
    :param send_date: Date the gift was sent in Unix time
    :type send_date: :obj:`int`
    :param is_saved: Optional. True, if the gift is displayed on the account's profile page
    :type is_saved: :obj:`bool`
    :param can_be_transferred: Optional. True, if the gift can be transferred to another owner
    :type can_be_transferred: :obj:`bool`
    :param transfer_star_count: Optional. Number of Telegram Stars that must be paid to transfer the gift
    :type transfer_star_count: :obj:`int`
    """

    def __init__(
        self,
        gift: "tgram.types.UniqueGift" = None,
        owned_gift_id: str = None,
        sender_user: "tgram.types.User" = None,
        send_date: int = None,
        is_saved: bool = None,
        can_be_transferred: bool = None,
        transfer_star_count: int = None,
        next_transfer_date: int = None,
        me: "tgram.TgBot" = None,
        json: dict = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "unique"
        self.gift = gift
        self.owned_gift_id = owned_gift_id
        self.sender_user = sender_user
        self.send_date = send_date
        self.is_saved = is_saved
        self.can_be_transferred = can_be_transferred
        self.transfer_star_count = transfer_star_count
        self.next_transfer_date = next_transfer_date

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["OwnedGiftUnique"]:
        return (
            OwnedGiftUnique(
                gift=tgram.types.UniqueGift._parse(me, d.get("gift")),
                owned_gift_id=d.get("owned_gift_id"),
                sender_user=tgram.types.User._parse(me, d.get("sender_user")),
                send_date=d.get("send_date"),
                is_saved=d.get("is_saved"),
                can_be_transferred=d.get("can_be_transferred"),
                transfer_star_count=d.get("transfer_star_count"),
                next_transfer_date=d.get("next_transfer_date"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class OwnedGifts(Type_):
    """
    Contains the list of gifts received and owned by a user or a chat.

    :param total_count: The total number of gifts owned by the user or the chat
    :type total_count: :obj:`int`
    :param gifts: The list of gifts
    :type gifts: List[Union[:class:`OwnedGiftRegular`, :class:`OwnedGiftUnique`]]
    :param next_offset: Optional. Offset for the next request. If empty, then there are no more results
    :type next_offset: :obj:`str`
    """

    def __init__(
        self,
        total_count: int = None,
        gifts: List[Union["OwnedGiftRegular", "OwnedGiftUnique"]] = None,
        next_offset: str = None,
        me: "tgram.TgBot" = None,
        json: dict = None,
    ):
        super().__init__(me=me, json=json)
        self.total_count = total_count
        self.gifts = gifts
        self.next_offset = next_offset

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["OwnedGifts"]:
        return (
            OwnedGifts(
                total_count=d.get("total_count"),
                gifts=[
                    (
                        OwnedGiftRegular._parse(me, g)
                        if g.get("type") == "regular"
                        else OwnedGiftUnique._parse(me, g)
                    )
                    for g in d.get("gifts")
                ],
                next_offset=d.get("next_offset"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class SetMyName:
    async def set_my_name(
        self: "tgram.TgBot", name: str = None, language_code: str = None
    ) -> bool:
        """
        Use this method to change the bot's name. Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#setmyname

        :param name: Optional. New bot name; 0-64 characters. Pass an empty string to remove the dedicated name for the given language.
        :type name: :obj:`str`

        :param language_code: Optional. A two-letter ISO 639-1 language code. If empty, the name will be shown to all users for whose
            language there is no dedicated name.
        :type language_code: :obj:`str`

        :return: True on success.
        """

        result = await self(
            "setMyName",
            name=name,
            language_code=language_code,
        )
        return result["result"]



import tgram

from typing import Optional


class PreCheckoutQueryB:
    async def answer(
        self: "tgram.types.PreCheckoutQuery",
        ok: "bool" = True,
        error_message: "str" = None,
    ) -> bool:
        return await self._me.answer_pre_checkout_query(
            pre_checkout_query_id=self.id, ok=ok, error_message=error_message
        )

    @property
    def user(self: "tgram.types.PreCheckoutQuery") -> Optional["tgram.types.User"]:
        return self.from_user

    sender_user = user



import tgram
from .type_ import Type_

from typing import Optional


class ChatBoost(Type_):
    """
    This object contains information about a chat boost.

    Telegram documentation: https://core.telegram.org/bots/api#chatboost

    :param boost_id: Unique identifier of the boost
    :type boost_id: :obj:`str`

    :param add_date: Point in time (Unix timestamp) when the chat was boosted
    :type add_date: :obj:`int`

    :param expiration_date: Point in time (Unix timestamp) when the boost will automatically expire, unless the booster's Telegram Premium subscription is prolonged
    :type expiration_date: :obj:`int`

    :param source: Optional. Source of the added boost (made Optional for now due to API error)
    :type source: :class:`ChatBoostSource`

    :return: Instance of the class
    :rtype: :class:`ChatBoost`
    """

    def __init__(
        self,
        boost_id: "str" = None,
        add_date: "int" = None,
        expiration_date: "int" = None,
        source: "tgram.types.ChatBoostSource" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.boost_id = boost_id
        self.add_date = add_date
        self.expiration_date = expiration_date
        self.source = source

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.ChatBoost"]:
        return (
            ChatBoost(
                me=me,
                json=d,
                boost_id=d.get("boost_id"),
                add_date=d.get("add_date"),
                expiration_date=d.get("expiration_date"),
                source=tgram.types.ChatBoostSource._parse(me=me, d=d.get("source")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters
from pathlib import Path
from tgram.utils import get_file_path, convert_to_inline_keyboard_markup


class SendVideoNote:
    async def send_video_note(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        video_note: Union[Path, bytes, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        duration: int = None,
        length: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        As of v.4.0, Telegram clients support rounded square MPEG4 videos of up to 1 minute long.
        Use this method to send video messages. On success, the sent Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#sendvideonote

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param data: Video note to send. Pass a file_id as String to send a video note that exists on the Telegram servers (recommended)
            or upload a new video using multipart/form-data. Sending video notes by a URL is currently unsupported
        :type data: :obj:`str` or :class:`tgram.types.InputFile`

        :param duration: Duration of sent video in seconds
        :type duration: :obj:`int`

        :param length: Video width and height, i.e. diameter of the video message
        :type length: :obj:`int`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard
            or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param thumbnail: Thumbnail of the file sent; can be ignored if thumbnail generation for the file is supported server-side.
            The thumbnail should be in JPEG format and less than 200 kB in size. A thumbnail's width and height should not exceed 320.
            Ignored if the file is not uploaded using multipart/form-data. Thumbnails can't be reused and can be only uploaded as a new file,
            so you can pass “attach://<file_attach_name>” if the thumbnail was uploaded using multipart/form-data under <file_attach_name>.
        :type thumbnail: :obj:`str` or :class:`tgram.types.InputFile`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_thread_id: Identifier of a message thread, in which the video note will be sent
        :type message_thread_id: :obj:`int`

        :param thumb: Deprecated. Use thumbnail instead
        :type thumb: :obj:`str` or :class:`tgram.types.InputFile`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Identifier of a business connection, in which the message will be sent
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier of the message effect
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendVideoNote",
            chat_id=chat_id,
            video_note=get_file_path(video_note),
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            duration=duration,
            length=length,
            thumbnail=get_file_path(thumbnail),
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from typing import List
from typing import Union
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ParseMode

from tgram.utils import get_parse_mode, convert_to_inline_keyboard_markup


class EditMessageCaption:
    async def edit_message_caption(
        self: "tgram.TgBot",
        business_connection_id: str = None,
        chat_id: Union[int, str] = None,
        message_id: int = None,
        inline_message_id: str = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        show_caption_above_media: bool = None,
        reply_markup: InlineKeyboardMarkup = None,
    ) -> Union[Message, bool]:
        """
        Use this method to edit captions of messages.
        On success, if the edited message is not an inline message, the edited Message is returned, otherwise True is returned.

        Note that business messages that were not sent by the bot and do not contain an inline keyboard can only be edited within 48 hours from the time they were sent.
        """
        result = await self(
            "editMessageCaption",
            business_connection_id=business_connection_id,
            chat_id=chat_id,
            message_id=message_id,
            inline_message_id=inline_message_id,
            caption=caption,
            parse_mode=get_parse_mode(self, parse_mode),
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
        )
        return (
            Message._parse(me=self, d=result["result"])
            if isinstance(result["result"], dict)
            else result["result"]
        )



import html
import tgram
import re

from typing import List
from struct import unpack


class String(str):
    def __init__(self, *args) -> None:
        self._entities: List["tgram.types.MessageEntity"] = None
        super().__init__()

    def put(self, e: List["tgram.types.MessageEntity"] = None) -> "String":
        self._entities = e

        return self

    @property
    def markdown(self) -> str:
        return self if not self._entities else markdown_unparse(self, self._entities)

    @property
    def html(self) -> str:
        return self if not self._entities else html_unparse(self, self._entities)


SMP_RE = re.compile(r"[\U00010000-\U0010FFFF]")

BOLD_DELIM = "**"
ITALIC_DELIM = "__"
UNDERLINE_DELIM = "--"
STRIKE_DELIM = "~~"
SPOILER_DELIM = "||"
CODE_DELIM = "`"
PRE_DELIM = "```"
BLOCKQUOTE_DELIM = ">"

MARKDOWN_RE = re.compile(
    r"({d})|(!?)\[(.+?)\]\((.+?)\)".format(
        d="|".join(
            [
                "".join(i)
                for i in [
                    [rf"\{j}" for j in i]
                    for i in [
                        PRE_DELIM,
                        CODE_DELIM,
                        STRIKE_DELIM,
                        UNDERLINE_DELIM,
                        ITALIC_DELIM,
                        BOLD_DELIM,
                        SPOILER_DELIM,
                    ]
                ]
            ]
        )
    )
)

OPENING_TAG = "<{}>"
CLOSING_TAG = "</{}>"
URL_MARKUP = '<a href="{}">{}</a>'
EMOJI_MARKUP = "<emoji id={}>{}</emoji>"
FIXED_WIDTH_DELIMS = [CODE_DELIM, PRE_DELIM]


def add_surrogates(text: str) -> str:
    # Replace each SMP code point with a surrogate pair
    return SMP_RE.sub(
        lambda match:  # Split SMP in two surrogates
        "".join(chr(i) for i in unpack("<HH", match.group().encode("utf-16le"))),
        text,
    )


def remove_surrogates(text: str) -> str:
    # Replace each surrogate pair with a SMP code point
    return text.encode("utf-16", "surrogatepass").decode("utf-16")


def markdown_unparse(text: str, entities: List["tgram.types.MessageEntity"]):
    text = add_surrogates(text)

    entities_offsets = []

    for entity in entities:
        entity_type = entity.type.lower()
        start = entity.offset
        end = start + entity.length

        if entity_type == "bold":
            start_tag = end_tag = BOLD_DELIM
        elif entity_type == "italic":
            start_tag = end_tag = ITALIC_DELIM
        elif entity_type == "underline":
            start_tag = end_tag = UNDERLINE_DELIM
        elif entity_type == "strikethrough":
            start_tag = end_tag = STRIKE_DELIM
        elif entity_type == "code":
            start_tag = end_tag = CODE_DELIM
        elif entity_type == "pre":
            language = getattr(entity, "language", "") or ""
            start_tag = f"{PRE_DELIM}{language}\n"
            end_tag = f"\n{PRE_DELIM}"
        elif entity_type == "blockquote":
            start_tag = BLOCKQUOTE_DELIM + " "
            end_tag = ""
            blockquote_text = text[start:end]
            lines = blockquote_text.split("\n")
            last_length = 0
            for line in lines:
                if len(line) == 0 and last_length == end:
                    continue
                start_offset = start + last_length
                last_length = last_length + len(line)
                end_offset = start_offset + last_length
                entities_offsets.append(
                    (
                        start_tag,
                        start_offset,
                    )
                )
                entities_offsets.append(
                    (
                        end_tag,
                        end_offset,
                    )
                )
                last_length = last_length + 1
            continue
        elif entity_type == "spoiler":
            start_tag = end_tag = SPOILER_DELIM
        elif entity_type == "text_link":
            url = entity.url
            start_tag = "["
            end_tag = f"]({url})"
        elif entity_type == "text_mention":
            user = entity.user
            start_tag = "["
            end_tag = f"](tg://user?id={user.id})"
        elif entity_type == "custom_emoji":
            emoji_id = entity.custom_emoji_id
            start_tag = "!["
            end_tag = f"](tg://emoji?id={emoji_id})"
        else:
            continue

        entities_offsets.append(
            (
                start_tag,
                start,
            )
        )
        entities_offsets.append(
            (
                end_tag,
                end,
            )
        )

    entities_offsets = map(
        lambda x: x[1],
        sorted(
            enumerate(entities_offsets), key=lambda x: (x[1][1], x[0]), reverse=True
        ),
    )

    for entity, offset in entities_offsets:
        text = text[:offset] + entity + text[offset:]

    return remove_surrogates(text)


def html_unparse(text: str, entities: List["tgram.types.MessageEntity"]) -> str:
    def parse_one(entity: "tgram.types.MessageEntity"):
        """
        Parses a single entity and returns (start_tag, start), (end_tag, end)
        """
        entity_type = entity.type.lower()
        start = entity.offset
        end = start + entity.length

        if entity_type in (
            "bold",
            "italic",
            "underline",
            "strikethrough",
        ):
            name = entity_type[0]
            start_tag = f"<{name}>"
            end_tag = f"</{name}>"
        elif entity_type == "pre":
            name = entity_type
            language = getattr(entity, "language", "") or ""
            start_tag = f'<{name} language="{language}">' if language else f"<{name}>"
            end_tag = f"</{name}>"
        elif entity_type == "blockquote":
            name = entity_type
            expandable = getattr(entity, "expandable", False)
            start_tag = f"<{name}{' expandable' if expandable else ''}>"
            end_tag = f"</{name}>"
        elif entity_type in (
            "code",
            "spoiler",
        ):
            name = entity_type if entity_type == "code" else "tg-spoiler"
            start_tag = f"<{name}>"
            end_tag = f"</{name}>"
        elif entity_type == "text_link":
            url = entity.url
            start_tag = f'<a href="{url}">'
            end_tag = "</a>"
        elif entity_type == "text_mention":
            user = entity.user
            start_tag = f'<a href="tg://user?id={user.id}">'
            end_tag = "</a>"
        elif entity_type == "custom_emoji":
            custom_emoji_id = entity.custom_emoji_id
            start_tag = f'<emoji id="{custom_emoji_id}">'
            end_tag = "</emoji>"
        else:
            return

        return (start_tag, start), (end_tag, end)

    def recursive(entity_i: int) -> int:
        """
        Takes the index of the entity to start parsing from, returns the number of parsed entities inside it.
        Uses entities_offsets as a stack, pushing (start_tag, start) first, then parsing nested entities,
        and finally pushing (end_tag, end) to the stack.
        No need to sort at the end.
        """
        this = parse_one(entities[entity_i])
        if this is None:
            return 1
        (start_tag, start), (end_tag, end) = this
        entities_offsets.append((start_tag, start))
        internal_i = entity_i + 1
        # while the next entity is inside the current one, keep parsing
        while internal_i < len(entities) and entities[internal_i].offset < end:
            internal_i += recursive(internal_i)
        entities_offsets.append((end_tag, end))
        return internal_i - entity_i

    text = add_surrogates(text)

    entities_offsets = []

    # probably useless because entities are already sorted by telegram
    entities.sort(key=lambda e: (e.offset, -e.length))

    # main loop for first-level entities
    i = 0
    while i < len(entities):
        i += recursive(i)

    if entities_offsets:
        last_offset = entities_offsets[-1][1]
        # no need to sort, but still add entities starting from the end
        for entity, offset in reversed(entities_offsets):
            text = (
                text[:offset]
                + entity
                + html.escape(text[offset:last_offset])
                + text[last_offset:]
            )
            last_offset = offset

    return remove_surrogates(text)



import tgram
from .type_ import Type_
from typing import Optional


class DirectMessagePriceChanged(Type_):
    """
    Describes a service message about a change in the price of direct messages sent to a channel chat.

    :param are_direct_messages_enabled: True, if direct messages are enabled for the channel chat; false otherwise
    :type are_direct_messages_enabled: :obj:`bool`

    :param direct_message_star_count: Optional. The new number of Telegram Stars that must be paid by users for each direct message sent to the channel. Does not apply to users who have been exempted by administrators. Defaults to 0.
    :type direct_message_star_count: :obj:`int`
    """

    def __init__(
        self,
        are_direct_messages_enabled: bool,
        direct_message_star_count: Optional[int] = 0,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.are_direct_messages_enabled = are_direct_messages_enabled
        self.direct_message_star_count = direct_message_star_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.DirectMessagePriceChanged"]:
        return (
            DirectMessagePriceChanged(
                me=me,
                json=d,
                are_direct_messages_enabled=d.get("are_direct_messages_enabled"),
                direct_message_star_count=d.get("direct_message_star_count", 0),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode
from pathlib import Path
from tgram.utils import get_file_path, get_parse_mode, convert_to_inline_keyboard_markup


class SendVoice:
    async def send_voice(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        voice: Union[Path, bytes, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        duration: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send audio files, if you want Telegram clients to display the file as a playable voice message. For this to work, your audio must be in an .OGG file encoded with OPUS, or in .MP3 format, or in .M4A format (other formats may be sent as Audio or Document). On success, the sent Message is returned. Bots can currently send voice messages of up to 50 MB in size, this limit may be changed in the future.

        Telegram documentation: https://core.telegram.org/bots/api#sendvoice

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param voice: Audio file to send. Pass a file_id as String to send a file that exists on the Telegram servers (recommended),
            pass an HTTP URL as a String for Telegram to get a file from the Internet, or upload a new one using multipart/form-data.
        :type voice: :obj:`str` or :class:`tgram.types.InputFile`

        :param caption: Voice message caption, 0-1024 characters after entities parsing
        :type caption: :obj:`str`

        :param duration: Duration of the voice message in seconds
        :type duration: :obj:`int`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions
            to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param parse_mode: Mode for parsing entities in the voice message caption. See formatting options for more details.
        :type parse_mode: :obj:`str`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param caption_entities: A JSON-serialized list of special entities that appear in the caption, which can be specified instead of parse_mode
        :type caption_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_thread_id: Identifier of a message thread, in which the message will be sent
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Unique identifier for the target business connection
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier for the message effect
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        """

        result = await self(
            "sendVoice",
            chat_id=chat_id,
            voice=get_file_path(voice),
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=get_parse_mode(self, parse_mode),
            caption_entities=caption_entities,
            duration=duration,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters

from tgram.utils import convert_to_inline_keyboard_markup


class SendDice:
    async def send_dice(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        emoji: str = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send an animated emoji that will display a random value. On success, the sent Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#senddice

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param emoji: Emoji on which the dice throw animation is based. Currently, must be one of “🎲”, “🎯”, “🏀”, “⚽”, “🎳”, or “🎰”.
            Dice can have values 1-6 for “🎲”, “🎯” and “🎳”, values 1-5 for “🏀” and “⚽”, and values 1-64 for “🎰”. Defaults to “🎲”
        :type emoji: :obj:`str`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions
            to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param protect_content: Protects the contents of the sent message from forwarding
        :type protect_content: :obj:`bool`

        :param message_thread_id: The identifier of a message thread, unique within the chat to which the message with the thread identifier belongs
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Unique identifier for the target business connection
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier for the message effect
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendDice",
            chat_id=chat_id,
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            emoji=emoji,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_

from typing import List, Optional
from tgram import bound
from tgram.utils import String, message_origin_parse


class Message(Type_, bound.MessageB):
    """
    This object represents a message.

    Telegram Documentation: https://core.telegram.org/bots/api#message

    :param message_id: Unique message identifier inside this chat
    :type message_id: :obj:`int`

    :param message_thread_id: Optional. Unique identifier of a message thread to which the message belongs; for supergroups only
    :type message_thread_id: :obj:`int`

    :param from_user: Optional. Sender of the message; empty for messages sent to channels. For backward compatibility, the
        field contains a fake sender user in non-channel chats, if the message was sent on behalf of a chat.
    :type from_user: :class:`tgram.types.User`

    :param sender_chat: Optional. Sender of the message, sent on behalf of a chat. For example, the channel itself for
        channel posts, the supergroup itself for messages from anonymous group administrators, the linked channel for
        messages automatically forwarded to the discussion group. For backward compatibility, the field from contains a
        fake sender user in non-channel chats, if the message was sent on behalf of a chat.
    :type sender_chat: :class:`tgram.types.Chat`

    :param sender_boost_count: Optional. If the sender of the message boosted the chat, the number of boosts added by the user
    :type sender_boost_count: :obj:`int`

    :param sender_business_bot info: Optional. Information about the business bot that sent the message
    :type sender_business_bot_info: :class:`tgram.types.User`

    :param date: Date the message was sent in Unix time
    :type date: :obj:`int`

    :param business_connection_id: Optional. Unique identifier of the business connection from which the message was received. If non-empty,
        the message belongs to a chat of the corresponding business account that is independent from any potential bot chat which might share the same identifier.
    :type business_connection_id: :obj:`str`

    :param chat: Conversation the message belongs to
    :type chat: :class:`tgram.types.Chat`

    :forward_origin: Optional. For forwarded messages, information about the original message;
    :type forward_origin: :class:`tgram.types.MessageOrigin`

    :param is_topic_message: Optional. True, if the message is sent to a forum topic
    :type is_topic_message: :obj:`bool`

    :param is_automatic_forward: Optional. :obj:`bool`, if the message is a channel post that was automatically
        forwarded to the connected discussion group
    :type is_automatic_forward: :obj:`bool`

    :param reply_to_message: Optional. For replies, the original message. Note that the Message object in this field
        will not contain further reply_to_message fields even if it itself is a reply.
    :type reply_to_message: :class:`tgram.types.Message`

    :param external_reply: Optional. Information about the message that is being replied to, which may come from another chat or forum topic
    :type external_reply: :class:`tgram.types.ExternalReplyInfo`

    :param quote: Optional. For replies that quote part of the original message, the quoted part of the message
    :type quote: :class:`tgram.types.TextQuote`

    :param reply_to_story: Optional. For replies to a story, the original story
    :type reply_to_story: :class:`tgram.types.Story`

    :param via_bot: Optional. Bot through which the message was sent
    :type via_bot: :class:`tgram.types.User`

    :param edit_date: Optional. Date the message was last edited in Unix time
    :type edit_date: :obj:`int`

    :param has_protected_content: Optional. :obj:`bool`, if the message can't be forwarded
    :type has_protected_content: :obj:`bool`

    :param is_from_offline: Optional. True, if the message was sent by an implicit action, for example,
        as an away or a greeting business message, or as a scheduled message
    :type is_from_offline: :obj:`bool`

    :param media_group_id: Optional. The unique identifier of a media message group this message belongs to
    :type media_group_id: :obj:`str`

    :param author_signature: Optional. Signature of the post author for messages in channels, or the custom title of an
        anonymous group administrator
    :type author_signature: :obj:`str`

    :param text: Optional. For text messages, the actual UTF-8 text of the message
    :type text: :class:`tgram.utils.String`

    :param entities: Optional. For text messages, special entities like usernames, URLs, bot commands, etc. that
        appear in the text
    :type entities: :obj:`list` of :class:`tgram.types.MessageEntity`

    :param link_preview_options: Optional. Options used for link preview generation for the message,
        if it is a text message and link preview options were changed
    :type link_preview_options: :class:`tgram.types.LinkPreviewOptions`

    :param effect_id: Optional. Unique identifier of the message effect added to the message
    :type effect_id: :obj:`str`

    :param animation: Optional. Message is an animation, information about the animation. For backward
        compatibility, when this field is set, the document field will also be set
    :type animation: :class:`tgram.types.Animation`

    :param audio: Optional. Message is an audio file, information about the file
    :type audio: :class:`tgram.types.Audio`

    :param document: Optional. Message is a general file, information about the file
    :type document: :class:`tgram.types.Document`

    :param photo: Optional. Message is a photo, available sizes of the photo
    :type photo: :obj:`list` of :class:`tgram.types.PhotoSize`

    :param sticker: Optional. Message is a sticker, information about the sticker
    :type sticker: :class:`tgram.types.Sticker`

    :param story: Optional. Message is a forwarded story
    :type story: :class:`tgram.types.Story`

    :param video: Optional. Message is a video, information about the video
    :type video: :class:`tgram.types.Video`

    :param video_note: Optional. Message is a video note, information about the video message
    :type video_note: :class:`tgram.types.VideoNote`

    :param voice: Optional. Message is a voice message, information about the file
    :type voice: :class:`tgram.types.Voice`

    :param caption: Optional. Caption for the animation, audio, document, photo, video or voice
    :type caption: :obj:`str`

    :param caption_entities: Optional. For messages with a caption, special entities like usernames, URLs, bot
        commands, etc. that appear in the caption
    :type caption_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

    :param show_caption_above_media: Optional. True, if the caption must be shown above the message media
    :type show_caption_above_media: :obj:`bool`

    :param has_media_spoiler: Optional. True, if the message media is covered by a spoiler animation
    :type has_media_spoiler: :obj:`bool`

    :param checklist: Optional. Message is a checklist
    :type checklist: :class:`tgram.types.Checklist`

    :param contact: Optional. Message is a shared contact, information about the contact
    :type contact: :class:`tgram.types.Contact`

    :param dice: Optional. Message is a dice with random value
    :type dice: :class:`tgram.types.Dice`

    :param game: Optional. Message is a game, information about the game. More about games »
    :type game: :class:`tgram.types.Game`

    :param poll: Optional. Message is a native poll, information about the poll
    :type poll: :class:`tgram.types.Poll`

    :param venue: Optional. Message is a venue, information about the venue. For backward compatibility, when this
        field is set, the location field will also be set
    :type venue: :class:`tgram.types.Venue`

    :param location: Optional. Message is a shared location, information about the location
    :type location: :class:`tgram.types.Location`

    :param new_chat_members: Optional. New members that were added to the group or supergroup and information about
        them (the bot itself may be one of these members)
    :type new_chat_members: :obj:`list` of :class:`tgram.types.User`

    :param left_chat_member: Optional. A member was removed from the group, information about them (this member may be
        the bot itself)
    :type left_chat_member: :class:`tgram.types.User`

    :param new_chat_title: Optional. A chat title was changed to this value
    :type new_chat_title: :obj:`str`

    :param new_chat_photo: Optional. A chat photo was change to this value
    :type new_chat_photo: :obj:`list` of :class:`tgram.types.PhotoSize`

    :param delete_chat_photo: Optional. Service message: the chat photo was deleted
    :type delete_chat_photo: :obj:`bool`

    :param group_chat_created: Optional. Service message: the group has been created
    :type group_chat_created: :obj:`bool`

    :param supergroup_chat_created: Optional. Service message: the supergroup has been created. This field can't be
        received in a message coming through updates, because bot can't be a member of a supergroup when it is created. It can
        only be found in reply_to_message if someone replies to a very first message in a directly created supergroup.
    :type supergroup_chat_created: :obj:`bool`

    :param channel_chat_created: Optional. Service message: the channel has been created. This field can't be
        received in a message coming through updates, because bot can't be a member of a channel when it is created. It can only
        be found in reply_to_message if someone replies to a very first message in a channel.
    :type channel_chat_created: :obj:`bool`

    :param message_auto_delete_timer_changed: Optional. Service message: auto-delete timer settings changed in
        the chat
    :type message_auto_delete_timer_changed: :class:`tgram.types.MessageAutoDeleteTimerChanged`

    :param migrate_to_chat_id: Optional. The group has been migrated to a supergroup with the specified identifier.
        This number may have more than 32 significant bits and some programming languages may have difficulty/silent
        defects in interpreting it. But it has at most 52 significant bits, so a signed 64-bit integer or double-precision
        float type are safe for storing this identifier.
    :type migrate_to_chat_id: :obj:`int`

    :param migrate_from_chat_id: Optional. The supergroup has been migrated from a group with the specified
        identifier. This number may have more than 32 significant bits and some programming languages may have
        difficulty/silent defects in interpreting it. But it has at most 52 significant bits, so a signed 64-bit integer or
        double-precision float type are safe for storing this identifier.
    :type migrate_from_chat_id: :obj:`int`

    :param pinned_message: Optional. Specified message was pinned. Note that the Message object in this field will not
        contain further reply_to_message fields even if it is itself a reply.
    :type pinned_message: :class:`tgram.types.Message` or :class:`tgram.types.InaccessibleMessage`

    :param invoice: Optional. Message is an invoice for a payment, information about the invoice. More about payments »
    :type invoice: :class:`tgram.types.Invoice`

    :param successful_payment: Optional. Message is a service message about a successful payment, information about
        the payment. More about payments »
    :type successful_payment: :class:`tgram.types.SuccessfulPayment`

    :param users_shared: Optional. Service message: a user was shared with the bot
    :type users_shared: :class:`tgram.types.UsersShared`

    :param chat_shared: Optional. Service message: a chat was shared with the bot
    :type chat_shared: :class:`tgram.types.ChatShared`

    :param gift: Optional. Service message: a regular gift was sent or received
        :type gift: :class:`tgram.types.GiftInfo`

    :param unique_gift: Optional. Service message: a unique gift was sent or received
    :type unique_gift: :class:`tgram.types.UniqueGiftInfo`

    :param connected_website: Optional. The domain name of the website on which the user has logged in. More about
        Telegram Login »
    :type connected_website: :obj:`str`

    :param write_access_allowed: Optional. Service message: the user allowed the bot added to the attachment
        menu to write messages
    :type write_access_allowed: :class:`tgram.types.WriteAccessAllowed`

    :param passport_data: Optional. Telegram Passport data
    :type passport_data: :class:`tgram.types.PassportData`

    :param proximity_alert_triggered: Optional. Service message. A user in the chat triggered another user's
        proximity alert while sharing Live Location.
    :type proximity_alert_triggered: :class:`tgram.types.ProximityAlertTriggered`

    :param boost_added: Optional. Service message: user boosted the chat
    :type boost_added: :class:`tgram.types.ChatBoostAdded`

    :param chat_background_set: Optional. Service message: chat background set
    :type chat_background_set: :class:`tgram.types.ChatBackground`

    :param checklist_tasks_done: Optional. Service message: some tasks in a checklist were marked as done or not done
    :type checklist_tasks_done: :class:`tgram.types.ChecklistTasksDone`

    :param checklist_tasks_added: Optional. Service message: tasks were added to a checklist
    :type checklist_tasks_added: :class:`tgram.types.ChecklistTasksAdded`

    :param direct_message_price_changed: Optional. Service message: the price for paid messages in the corresponding direct messages chat of a channel has changed
    :type direct_message_price_changed: :class:`tgram.types.DirectMessagePriceChanged`

    :param forum_topic_created: Optional. Service message: forum topic created
    :type forum_topic_created: :class:`tgram.types.ForumTopicCreated`

    :param forum_topic_edited: Optional. Service message: forum topic edited
    :type forum_topic_edited: :class:`tgram.types.ForumTopicEdited`

    :param forum_topic_closed: Optional. Service message: forum topic closed
    :type forum_topic_closed: :class:`tgram.types.ForumTopicClosed`

    :param forum_topic_reopened: Optional. Service message: forum topic reopened
    :type forum_topic_reopened: :class:`tgram.types.ForumTopicReopened`

    :param general_forum_topic_hidden: Optional. Service message: the 'General' forum topic hidden
    :type general_forum_topic_hidden: :class:`tgram.types.GeneralForumTopicHidden`

    :param general_forum_topic_unhidden: Optional. Service message: the 'General' forum topic unhidden
    :type general_forum_topic_unhidden: :class:`tgram.types.GeneralForumTopicUnhidden`

    :param giveaway_created: Optional. Service message: a giveaway has been created
    :type giveaway_created: :class:`tgram.types.GiveawayCreated`

    :param giveaway: Optional. The message is a scheduled giveaway message
    :type giveaway: :class:`tgram.types.Giveaway`

    :param giveaway_winners: Optional. Service message: giveaway winners(public winners)
    :type giveaway_winners: :class:`tgram.types.GiveawayWinners`

    :param giveaway_completed: Optional. Service message: giveaway completed, without public winners
    :type giveaway_completed: :class:`tgram.types.GiveawayCompleted`

    :param video_chat_scheduled: Optional. Service message: video chat scheduled
    :type video_chat_scheduled: :class:`tgram.types.VideoChatScheduled`

    :param video_chat_started: Optional. Service message: video chat started
    :type video_chat_started: :class:`tgram.types.VideoChatStarted`

    :param video_chat_ended: Optional. Service message: video chat ended
    :type video_chat_ended: :class:`tgram.types.VideoChatEnded`

    :param video_chat_participants_invited: Optional. Service message: new participants invited to a video chat
    :type video_chat_participants_invited: :class:`tgram.types.VideoChatParticipantsInvited`

    :param web_app_data: Optional. Service message: data sent by a Web App
    :type web_app_data: :class:`tgram.types.WebAppData`

    :param reply_markup: Optional. Inline keyboard attached to the message. login_url buttons are represented as ordinary url buttons.
    :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup`

    :param paid_message_price_changed: Optional. Service message: the price for paid messages has changed in the chat
    :type paid_message_price_changed: :class:`tgram.types.PaidMessagePriceChanged`

    :return: Instance of the class
    :rtype: :class:`tgram.types.Message`
    """

    def __init__(
        self,
        message_id: "int" = None,
        date: "int" = None,
        chat: "tgram.types.Chat" = None,
        message_thread_id: "int" = None,
        from_user: "tgram.types.User" = None,
        sender_chat: "tgram.types.Chat" = None,
        sender_boost_count: "int" = None,
        sender_business_bot: "tgram.types.User" = None,
        business_connection_id: "str" = None,
        forward_origin: "tgram.types.MessageOrigin" = None,
        is_topic_message: "bool" = None,
        is_automatic_forward: "bool" = None,
        reply_to_message: "tgram.types.Message" = None,
        external_reply: "tgram.types.ExternalReplyInfo" = None,
        quote: "tgram.types.TextQuote" = None,
        reply_to_story: "tgram.types.Story" = None,
        via_bot: "tgram.types.User" = None,
        edit_date: "int" = None,
        has_protected_content: "bool" = None,
        is_from_offline: "bool" = None,
        media_group_id: "str" = None,
        author_signature: "str" = None,
        text: "String" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        effect_id: "str" = None,
        animation: "tgram.types.Animation" = None,
        audio: "tgram.types.Audio" = None,
        document: "tgram.types.Document" = None,
        paid_media: "tgram.types.PaidMediaInfo" = None,
        photo: List["tgram.types.PhotoSize"] = None,
        sticker: "tgram.types.Sticker" = None,
        story: "tgram.types.Story" = None,
        video: "tgram.types.Video" = None,
        video_note: "tgram.types.VideoNote" = None,
        voice: "tgram.types.Voice" = None,
        caption: "String" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: "bool" = None,
        has_media_spoiler: "bool" = None,
        checklist: "tgram.types.Checklist" = None,
        contact: "tgram.types.Contact" = None,
        dice: "tgram.types.Dice" = None,
        game: "tgram.types.Game" = None,
        poll: "tgram.types.Poll" = None,
        venue: "tgram.types.Venue" = None,
        location: "tgram.types.Location" = None,
        new_chat_members: List["tgram.types.User"] = None,
        left_chat_member: "tgram.types.User" = None,
        new_chat_title: "str" = None,
        new_chat_photo: List["tgram.types.PhotoSize"] = None,
        delete_chat_photo: "bool" = None,
        group_chat_created: "bool" = None,
        supergroup_chat_created: "bool" = None,
        channel_chat_created: "bool" = None,
        message_auto_delete_timer_changed: "tgram.types.MessageAutoDeleteTimerChanged" = None,
        migrate_to_chat_id: "int" = None,
        migrate_from_chat_id: "int" = None,
        pinned_message: "tgram.types.Message" = None,
        invoice: "tgram.types.Invoice" = None,
        successful_payment: "tgram.types.SuccessfulPayment" = None,
        refunded_payment: "tgram.types.RefundedPayment" = None,
        users_shared: "tgram.types.UsersShared" = None,
        chat_shared: "tgram.types.ChatShared" = None,
        gift: "tgram.types.GiftInfo" = None,
        unique_gift: "tgram.types.UniqueGiftInfo" = None,
        connected_website: "str" = None,
        write_access_allowed: "tgram.types.WriteAccessAllowed" = None,
        passport_data: "tgram.types.PassportData" = None,
        proximity_alert_triggered: "tgram.types.ProximityAlertTriggered" = None,
        boost_added: "tgram.types.ChatBoostAdded" = None,
        chat_background_set: "tgram.types.ChatBackground" = None,
        checklist_tasks_done: "tgram.types.ChecklistTasksDone" = None,
        checklist_tasks_added: "tgram.types.ChecklistTasksAdded" = None,
        direct_message_price_changed: "tgram.types.DirectMessagePriceChanged" = None,
        forum_topic_created: "tgram.types.ForumTopicCreated" = None,
        forum_topic_edited: "tgram.types.ForumTopicEdited" = None,
        forum_topic_closed: "tgram.types.ForumTopicClosed" = None,
        forum_topic_reopened: "tgram.types.ForumTopicReopened" = None,
        general_forum_topic_hidden: "tgram.types.GeneralForumTopicHidden" = None,
        general_forum_topic_unhidden: "tgram.types.GeneralForumTopicUnhidden" = None,
        giveaway_created: "tgram.types.GiveawayCreated" = None,
        giveaway: "tgram.types.Giveaway" = None,
        giveaway_winners: "tgram.types.GiveawayWinners" = None,
        giveaway_completed: "tgram.types.GiveawayCompleted" = None,
        paid_message_price_changed: "tgram.types.PaidMessagePriceChanged" = None,
        video_chat_scheduled: "tgram.types.VideoChatScheduled" = None,
        video_chat_started: "tgram.types.VideoChatStarted" = None,
        video_chat_ended: "tgram.types.VideoChatEnded" = None,
        video_chat_participants_invited: "tgram.types.VideoChatParticipantsInvited" = None,
        web_app_data: "tgram.types.WebAppData" = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.message_id = message_id
        self.message_thread_id = message_thread_id
        self.from_user = from_user
        self.sender_chat = sender_chat
        self.sender_boost_count = sender_boost_count
        self.sender_business_bot = sender_business_bot
        self.date = date
        self.business_connection_id = business_connection_id
        self.chat = chat
        self.forward_origin = forward_origin
        self.is_topic_message = is_topic_message
        self.is_automatic_forward = is_automatic_forward
        self.reply_to_message = reply_to_message
        self.external_reply = external_reply
        self.quote = quote
        self.reply_to_story = reply_to_story
        self.via_bot = via_bot
        self.edit_date = edit_date
        self.has_protected_content = has_protected_content
        self.is_from_offline = is_from_offline
        self.media_group_id = media_group_id
        self.author_signature = author_signature
        self.text = String(text).put(entities) if text else None
        self.entities = entities
        self.link_preview_options = link_preview_options
        self.effect_id = effect_id
        self.animation = animation
        self.audio = audio
        self.document = document
        self.paid_media = paid_media
        self.photo = photo
        self.sticker = sticker
        self.story = story
        self.video = video
        self.video_note = video_note
        self.voice = voice
        self.caption = String(caption).put(caption_entities) if caption else None
        self.caption_entities = caption_entities
        self.show_caption_above_media = show_caption_above_media
        self.has_media_spoiler = has_media_spoiler
        self.checklist = checklist
        self.contact = contact
        self.dice = dice
        self.game = game
        self.poll = poll
        self.venue = venue
        self.location = location
        self.new_chat_members = new_chat_members
        self.left_chat_member = left_chat_member
        self.new_chat_title = new_chat_title
        self.new_chat_photo = new_chat_photo
        self.delete_chat_photo = delete_chat_photo
        self.group_chat_created = group_chat_created
        self.supergroup_chat_created = supergroup_chat_created
        self.channel_chat_created = channel_chat_created
        self.message_auto_delete_timer_changed = message_auto_delete_timer_changed
        self.migrate_to_chat_id = migrate_to_chat_id
        self.migrate_from_chat_id = migrate_from_chat_id
        self.pinned_message = pinned_message
        self.invoice = invoice
        self.successful_payment = successful_payment
        self.refunded_payment = refunded_payment
        self.users_shared = users_shared
        self.chat_shared = chat_shared
        self.gift = gift
        self.unique_gift = unique_gift
        self.connected_website = connected_website
        self.write_access_allowed = write_access_allowed
        self.passport_data = passport_data
        self.proximity_alert_triggered = proximity_alert_triggered
        self.boost_added = boost_added
        self.chat_background_set = chat_background_set
        self.checklist_tasks_done = checklist_tasks_done
        self.checklist_tasks_added = checklist_tasks_added
        self.direct_message_price_changed = direct_message_price_changed
        self.forum_topic_created = forum_topic_created
        self.forum_topic_edited = forum_topic_edited
        self.forum_topic_closed = forum_topic_closed
        self.forum_topic_reopened = forum_topic_reopened
        self.general_forum_topic_hidden = general_forum_topic_hidden
        self.general_forum_topic_unhidden = general_forum_topic_unhidden
        self.giveaway_created = giveaway_created
        self.giveaway = giveaway
        self.giveaway_winners = giveaway_winners
        self.giveaway_completed = giveaway_completed
        self.paid_message_price_changed = paid_message_price_changed
        self.video_chat_scheduled = video_chat_scheduled
        self.video_chat_started = video_chat_started
        self.video_chat_ended = video_chat_ended
        self.video_chat_participants_invited = video_chat_participants_invited
        self.web_app_data = web_app_data
        self.reply_markup = reply_markup

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.Message"]:
        return (
            Message(
                me=me,
                json=d,
                message_id=d.get("message_id"),
                date=d.get("date"),
                chat=tgram.types.Chat._parse(me=me, d=d.get("chat")),
                message_thread_id=d.get("message_thread_id"),
                from_user=tgram.types.User._parse(me=me, d=d.get("from")),
                sender_chat=tgram.types.Chat._parse(me=me, d=d.get("sender_chat")),
                sender_boost_count=d.get("sender_boost_count"),
                sender_business_bot=tgram.types.User._parse(
                    me=me, d=d.get("sender_business_bot")
                ),
                business_connection_id=d.get("business_connection_id"),
                forward_origin=message_origin_parse(d.get("forward_origin"), me),
                is_topic_message=d.get("is_topic_message"),
                is_automatic_forward=d.get("is_automatic_forward"),
                reply_to_message=tgram.types.Message._parse(
                    me=me, d=d.get("reply_to_message")
                ),
                external_reply=tgram.types.ExternalReplyInfo._parse(
                    me=me, d=d.get("external_reply")
                ),
                quote=tgram.types.TextQuote._parse(me=me, d=d.get("quote")),
                reply_to_story=tgram.types.Story._parse(
                    me=me, d=d.get("reply_to_story")
                ),
                via_bot=tgram.types.User._parse(me=me, d=d.get("via_bot")),
                edit_date=d.get("edit_date"),
                has_protected_content=d.get("has_protected_content"),
                is_from_offline=d.get("is_from_offline"),
                media_group_id=d.get("media_group_id"),
                author_signature=d.get("author_signature"),
                text=d.get("text"),
                entities=[
                    tgram.types.MessageEntity._parse(me=me, d=i)
                    for i in d.get("entities")
                ]
                if d.get("entities")
                else None,
                link_preview_options=tgram.types.LinkPreviewOptions._parse(
                    me=me, d=d.get("link_preview_options")
                ),
                effect_id=d.get("effect_id"),
                animation=tgram.types.Animation._parse(me=me, d=d.get("animation")),
                audio=tgram.types.Audio._parse(me=me, d=d.get("audio")),
                document=tgram.types.Document._parse(me=me, d=d.get("document")),
                paid_media=tgram.types.PaidMediaInfo._parse(
                    me=me, d=d.get("paid_media")
                ),
                photo=[tgram.types.PhotoSize._parse(me=me, d=i) for i in d.get("photo")]
                if d.get("photo")
                else None,
                sticker=tgram.types.Sticker._parse(me=me, d=d.get("sticker")),
                story=tgram.types.Story._parse(me=me, d=d.get("story")),
                video=tgram.types.Video._parse(me=me, d=d.get("video")),
                video_note=tgram.types.VideoNote._parse(me=me, d=d.get("video_note")),
                voice=tgram.types.Voice._parse(me=me, d=d.get("voice")),
                caption=d.get("caption"),
                caption_entities=[
                    tgram.types.MessageEntity._parse(me=me, d=i)
                    for i in d.get("caption_entities")
                ]
                if d.get("caption_entities")
                else None,
                show_caption_above_media=d.get("show_caption_above_media"),
                has_media_spoiler=d.get("has_media_spoiler"),
                checklist=tgram.types.Checklist._parse(me=me, d=d.get("checklist")),
                contact=tgram.types.Contact._parse(me=me, d=d.get("contact")),
                dice=tgram.types.Dice._parse(me=me, d=d.get("dice")),
                game=tgram.types.Game._parse(me=me, d=d.get("game")),
                poll=tgram.types.Poll._parse(me=me, d=d.get("poll")),
                venue=tgram.types.Venue._parse(me=me, d=d.get("venue")),
                location=tgram.types.Location._parse(me=me, d=d.get("location")),
                new_chat_members=[
                    tgram.types.User._parse(me=me, d=i)
                    for i in d.get("new_chat_members")
                ]
                if d.get("new_chat_members")
                else None,
                left_chat_member=tgram.types.User._parse(
                    me=me, d=d.get("left_chat_member")
                ),
                new_chat_title=d.get("new_chat_title"),
                new_chat_photo=[
                    tgram.types.PhotoSize._parse(me=me, d=i)
                    for i in d.get("new_chat_photo")
                ]
                if d.get("new_chat_photo")
                else None,
                delete_chat_photo=d.get("delete_chat_photo"),
                group_chat_created=d.get("group_chat_created"),
                supergroup_chat_created=d.get("supergroup_chat_created"),
                channel_chat_created=d.get("channel_chat_created"),
                message_auto_delete_timer_changed=tgram.types.MessageAutoDeleteTimerChanged._parse(
                    me=me, d=d.get("message_auto_delete_timer_changed")
                ),
                migrate_to_chat_id=d.get("migrate_to_chat_id"),
                migrate_from_chat_id=d.get("migrate_from_chat_id"),
                pinned_message=tgram.types.Message._parse(
                    me=me, d=d.get("pinned_message")
                ),
                invoice=tgram.types.Invoice._parse(me=me, d=d.get("invoice")),
                successful_payment=tgram.types.SuccessfulPayment._parse(
                    me=me, d=d.get("successful_payment")
                ),
                refunded_payment=tgram.types.RefundedPayment._parse(
                    me=me, d=d.get("refunded_payment")
                ),
                users_shared=tgram.types.UsersShared._parse(
                    me=me, d=d.get("users_shared")
                ),
                chat_shared=tgram.types.ChatShared._parse(
                    me=me, d=d.get("chat_shared")
                ),
                gift=tgram.types.GiftInfo._parse(me=me, d=d.get("gift")),
                unique_gift=tgram.types.UniqueGiftInfo._parse(
                    me=me, d=d.get("unique_gift")
                ),
                connected_website=d.get("connected_website"),
                write_access_allowed=tgram.types.WriteAccessAllowed._parse(
                    me=me, d=d.get("write_access_allowed")
                ),
                passport_data=tgram.types.PassportData._parse(
                    me=me, d=d.get("passport_data")
                ),
                proximity_alert_triggered=tgram.types.ProximityAlertTriggered._parse(
                    me=me, d=d.get("proximity_alert_triggered")
                ),
                boost_added=tgram.types.ChatBoostAdded._parse(
                    me=me, d=d.get("boost_added")
                ),
                chat_background_set=tgram.types.ChatBackground._parse(
                    me=me, d=d.get("chat_background_set")
                ),
                checklist_tasks_done=tgram.types.ChecklistTasksDone._parse(
                    me=me, d=d.get("checklist_tasks_done")
                ),
                checklist_tasks_added=tgram.types.ChecklistTasksAdded._parse(
                    me=me, d=d.get("checklist_tasks_added")
                ),
                direct_message_price_changed=tgram.types.DirectMessagePriceChanged._parse(
                    me=me, d=d.get("direct_message_price_changed")
                ),
                forum_topic_created=tgram.types.ForumTopicCreated._parse(
                    me=me, d=d.get("forum_topic_created")
                ),
                forum_topic_edited=tgram.types.ForumTopicEdited._parse(
                    me=me, d=d.get("forum_topic_edited")
                ),
                forum_topic_closed=tgram.types.ForumTopicClosed._parse(
                    me=me, d=d.get("forum_topic_closed")
                ),
                forum_topic_reopened=tgram.types.ForumTopicReopened._parse(
                    me=me, d=d.get("forum_topic_reopened")
                ),
                general_forum_topic_hidden=tgram.types.GeneralForumTopicHidden._parse(
                    me=me, d=d.get("general_forum_topic_hidden")
                ),
                general_forum_topic_unhidden=tgram.types.GeneralForumTopicUnhidden._parse(
                    me=me, d=d.get("general_forum_topic_unhidden")
                ),
                giveaway_created=tgram.types.GiveawayCreated._parse(
                    me=me, d=d.get("giveaway_created")
                ),
                giveaway=tgram.types.Giveaway._parse(me=me, d=d.get("giveaway")),
                giveaway_winners=tgram.types.GiveawayWinners._parse(
                    me=me, d=d.get("giveaway_winners")
                ),
                giveaway_completed=tgram.types.GiveawayCompleted._parse(
                    me=me, d=d.get("giveaway_completed")
                ),
                paid_message_price_changed=tgram.types.PaidMessagePriceChanged._parse(
                    me=me, d=d.get("paid_message_price_changed")
                ),
                video_chat_scheduled=tgram.types.VideoChatScheduled._parse(
                    me=me, d=d.get("video_chat_scheduled")
                ),
                video_chat_started=tgram.types.VideoChatStarted._parse(
                    me=me, d=d.get("video_chat_started")
                ),
                video_chat_ended=tgram.types.VideoChatEnded._parse(
                    me=me, d=d.get("video_chat_ended")
                ),
                video_chat_participants_invited=tgram.types.VideoChatParticipantsInvited._parse(
                    me=me, d=d.get("video_chat_participants_invited")
                ),
                web_app_data=tgram.types.WebAppData._parse(
                    me=me, d=d.get("web_app_data")
                ),
                reply_markup=tgram.types.InlineKeyboardMarkup._parse(
                    me=me, d=d.get("reply_markup")
                ),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from tgram.types import File


class GetFile:
    async def get_file(self: "tgram.TgBot", file_id: str) -> File:
        """
        Use this method to get basic info about a file and prepare it for downloading.
        For the moment, bots can download files of up to 20MB in size.
        On success, a File object is returned.
        It is guaranteed that the link will be valid for at least 1 hour.
        When the link expires, a new one can be requested by calling get_file again.

        Telegram documentation: https://core.telegram.org/bots/api#getfile

        :param file_id: File identifier
        :type file_id: :obj:`str`

        :return: :class:`tgram.types.File`
        """

        result = await self(
            "getFile",
            file_id=file_id,
        )
        return File._parse(me=self, d=result["result"])



import tgram

from typing import Optional


class ShippingQueryB:
    @property
    def user(self: "tgram.types.ShippingQuery") -> Optional["tgram.types.User"]:
        return self.from_user

    sender_user = user



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode

from tgram import utils


class SendMediaFromFileId:
    async def send_media_from_file_id(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        file_id: str,
        business_connection_id: str = None,
        message_thread_id: int = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        show_caption_above_media: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        decoded_file_id = utils.decode_file_id(file_id)

        if decoded_file_id["file_type_int"] not in utils.SUPPORTED_FILE_TYPES_TO_SEND:
            raise ValueError(
                f"Unsupported file type to send {decoded_file_id['file_type']}, You have to download it first."
            )

        result = await self(
            "send" + decoded_file_id["file_type"],
            **{
                "chat_id": chat_id,
                decoded_file_id["file_type"].lower(): file_id,
                "business_connection_id": business_connection_id,
                "message_thread_id": message_thread_id,
                "caption": caption,
                "parse_mode": parse_mode or self.parse_mode,
                "caption_entities": caption_entities,
                "show_caption_above_media": show_caption_above_media,
                "disable_notification": disable_notification,
                "protect_content": protect_content
                if protect_content is not None
                else self.protect_content,
                "message_effect_id": message_effect_id,
                "reply_parameters": reply_parameters,
                "reply_markup": utils.convert_to_inline_keyboard_markup(reply_markup)
                if isinstance(reply_markup, list)
                else reply_markup,
                "allow_paid_broadcast": allow_paid_broadcast,
            },
        )

        return Message._parse(me=self, d=result["result"])



import tgram


class SetBusinessAccountBio:
    async def set_business_account_bio(
        self: "tgram.TgBot",
        business_connection_id: str,
        bio: str = None,
    ) -> bool:
        """
        Changes the bio of a managed business account.
        Requires the can_change_bio business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#setbusinessaccountbio

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param bio: The new value of the bio for the business account; 0-140 characters
        :type bio: :obj:`str`, optional

        :return: On success, returns True.
        :rtype: :obj:`bool`
        """

        result = await self(
            "setBusinessAccountBio",
            business_connection_id=business_connection_id,
            bio=bio,
        )
        return result["result"]



import tgram
import asyncio
import logging

from ..errors import MutedError
from ..storage.utils import check_update

from typing import Callable, Any
from collections import OrderedDict

logger = logging.getLogger(__name__)


class Dispatcher:
    async def handler_worker(self: "tgram.TgBot", lock: asyncio.Lock):
        while True:
            update = await self.updates_queue.get()

            if update is None:
                break

            async with lock:
                if isinstance(update, tgram.types.Update):
                    try:
                        await check_update(update)
                        await self._check_update(update)
                    except MutedError:
                        continue
                    except Exception as e:
                        logger.exception(e)
                elif isinstance(update, dict):
                    try:
                        await self._process_exception(
                            update["e"], update["m"], **update["kwargs"]
                        )
                    except Exception as e:
                        logger.exception(e)

    async def run_for_updates(self: "tgram.TgBot", skip_updates: bool = None) -> None:
        if self.plugins:
            self.load_plugins()

        # Determine offset, allowed updates, and limit based on skip_updates flag
        offset, allowed_updates, limit = (
            -1
            if (self.skip_updates if skip_updates is None else skip_updates)
            else None,
            self.allowed_updates,
            100,
        )
        self.is_running = True

        if not self._me:
            self._me = await self.get_me()

        # Create handler worker tasks
        for _ in range(self.workers):
            self.locks_list.append(asyncio.Lock())
            self.handler_worker_tasks.append(
                self.loop.create_task(self.handler_worker(self.locks_list[-1]))
            )

        logger.info("Started %s Handler Tasks", self.workers)

        while self.is_running:
            try:
                updates = await self.get_updates(
                    offset=offset,
                    limit=limit,
                    allowed_updates=allowed_updates,
                    timeout=55,
                )
                for update in updates:
                    offset = update.update_id + 1
                    self.updates_queue.put_nowait(update)
            except (asyncio.CancelledError, KeyboardInterrupt):
                self.is_running = False
            except asyncio.TimeoutError:
                continue
            except Exception as e:
                logger.exception(e)

        session = await self._get_session()
        await session.close()

    async def _check_cancel(
        self: "tgram.TgBot", callback: Callable, update: Any
    ) -> bool:
        logger.debug("Checking listener in %s func", callback.__name__)
        try:
            if asyncio.iscoroutinefunction(callback):
                return await callback(self, update)
            else:
                return await self.loop.run_in_executor(
                    self.executor, callback, self, update
                )
        except Exception as e:
            logger.exception(e)

    async def _check_update(self: "tgram.TgBot", update: "tgram.types.Update") -> None:
        for listener in self._listen_handlers:
            if (
                attr := getattr(update, listener.update_type)
            ) and await listener.filters(self, attr):
                self._listen_handlers.remove(listener)
                if listener.cancel is not None:
                    result = await self._check_cancel(listener.cancel, attr)
                    if result:
                        return
                return await self._process_listener(
                    attr, listener.next_step, listener.data
                )

        for group, group_items in self.groups.items():
            for handler in group_items:
                try:
                    if handler.type == "all":
                        await self._process_update(update, handler.callback, group)
                    elif handler.type == "exception":
                        continue
                    elif (
                        attr := getattr(update, handler.type)
                    ) and await handler.filter(self, attr):
                        await self._process_update(attr, handler.callback, group)
                except tgram.StopPropagation:
                    return
                except tgram.ContinuePropagation:
                    continue
                except Exception as e:
                    logger.exception(e)
                    continue

    async def _process_listener(
        self: "tgram.TgBot", update: Any, callback: Callable, data: dict
    ) -> None:
        logger.debug("Processing listener to %s func", callback.__name__)
        try:
            if asyncio.iscoroutinefunction(callback):
                await callback(self, update, data)
            else:
                await self.loop.run_in_executor(
                    self.executor, callback, self, update, data
                )
        except Exception as e:
            logger.exception(e)

    async def _process_update(
        self: "tgram.TgBot", update: Any, callback: Callable, group: int
    ) -> None:
        if hasattr(update, "_groups") and group in getattr(update, "_groups"):
            return
        if not hasattr(update, "_groups"):
            update._groups = []
        update._groups.append(group)
        logger.debug("Processing update to %s func", callback.__name__)
        try:
            if asyncio.iscoroutinefunction(callback):
                await callback(self, update)
            else:
                await self.loop.run_in_executor(self.executor, callback, self, update)
        except tgram.StopPropagation:
            raise
        except tgram.ContinuePropagation:
            update._groups.remove(group)
            raise
        except Exception as e:
            logger.exception(e)

    async def _process_exception(
        self: "tgram.TgBot", exception: Exception, method: str, **kwargs
    ) -> None:
        for group_items in self.groups.values():
            for handler in group_items:
                try:
                    if handler.type == "exception":
                        logger.debug(
                            "Processing exception to %s func", handler.callback.__name__
                        )
                        if asyncio.iscoroutinefunction(handler.callback):
                            await handler.callback(self, exception, method, **kwargs)
                        else:
                            await self.loop.run_in_executor(
                                self.executor,
                                handler.callback,
                                self,
                                exception,
                                method,
                                **kwargs,
                            )
                except Exception as e:
                    logger.exception(e)
                    continue

    async def _add_grouped_handler(
        self: "tgram.TgBot", handler: "tgram.handlers.Handler", group: int
    ):
        for lock in self.locks_list:
            await lock.acquire()

        try:
            if group not in self.groups:
                self.groups[group] = []
                self.groups = OrderedDict(sorted(self.groups.items()))

            self.groups[group].append(handler)
            logger.info(
                "(%s) added to %s handlers in group %s",
                handler.callback.__name__,
                "Update." + handler.type if handler.type != "all" else "all",
                group,
            )
        finally:
            for lock in self.locks_list:
                lock.release()

    async def _remove_grouped_handler(
        self: "tgram.TgBot", handler: "tgram.handlers.Handler", group: int
    ):
        for lock in self.locks_list:
            await lock.acquire()

        try:
            if group not in self.groups:
                raise ValueError(
                    f"Group {group} does not exist. Handler was not removed."
                )

            self.groups[group].remove(handler)
            logger.info(
                "(%s) removed from %s handlers from group %s",
                handler.callback.__name__,
                "Update." + handler.type if handler.type != "all" else "all",
                group,
            )
        finally:
            for lock in self.locks_list:
                lock.release()



import tgram
from .type_ import Type_

from typing import Optional


class TransactionPartnerTelegramApi(Type_):
    def __init__(
        self, request_count: int, me: "tgram.TgBot" = None, json: "dict" = None
    ):
        super().__init__(me=me, json=json)
        self.type = "telegram_api"
        self.request_count = request_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.TransactionPartnerTelegramApi"]:
        return (
            TransactionPartnerTelegramApi(
                me=me, json=d, request_count=d.get("request_count")
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )
